@@ -3,7 +3,7 @@ package threads::lite;
 use strict;
 use warnings;
 
-our $VERSION = '0.029_001';
+our $VERSION = '0.029_002';
 
 use 5.010;
 
@@ -13,9 +13,9 @@ use Storable 2.05 ();
 use XSLoader;
 XSLoader::load('threads::lite', $VERSION);
 
-our @EXPORT_OK   = qw/spawn receive_match receive receive_nb receive_table receive_table_nb self send_to/;
+our @EXPORT_OK   = qw/spawn receive receive_nb receive_match receive_match_nb self send_to/;
 our %EXPORT_TAGS = (
-	receive => [qw/receive receive_nb receive_table receive_table_nb/],
+	receive => [qw/receive receive_nb receive_match receive_match_nb/],
 	all     => \@EXPORT_OK,
 );
 
@@ -39,11 +39,11 @@ sub receive {
 	my @args = @_;
 	if (@args) {
 		for my $index (0..$#mailbox) {
-			return @{ splice @mailbox, $index, 1 } if $mailbox[$index] ~~ @args;
+			return _return_elements(splice @mailbox, $index, 1) if $mailbox[$index] ~~ @args;
 		}
 		while (1) {
 			my $message = _receive;
-			return @{$message} if @{$message} ~~ @args;
+			return _return_elements($message) if $message ~~ @args;
 			push @mailbox, $message;
 		}
 	}
@@ -56,16 +56,17 @@ sub receive_nb {
 	my @args = @_;
 	if (@args) {
 		for my $index (0..$#mailbox) {
-			return @{ splice @mailbox, $index, 1 } if $mailbox[$index] ~~ @args;
+			return _return_elements(splice @mailbox, $index, 1) if $mailbox[$index] ~~ @args;
 		}
 		while (my $message = _receive_nb) {
-			return @{$message} if @{$message} ~~ @args;
+			return _return_elements($message) if $message ~~ @args;
 			push @mailbox, $message;
 		}
 		return;
 	}
 	else {
-		return _return_elements(@mailbox ? shift @mailbox : _receive_nb);
+		my $ret = @mailbox ? shift @mailbox : _receive_nb;
+		return $ret ? _return_elements($ret) : $ret;
 	}
 }
 
@@ -91,7 +92,7 @@ sub receive_match(&) {
 		for ($message) {
 			$receive->();
 			@save = ($message);
-			redo MESSAGE;
+			next MESSAGE;
 		}
 		continue {
 			return _return_elements($message);
@@ -113,14 +114,14 @@ sub receive_match_nb(&) {
 		}
 		else {
 			push @mailbox, @save;
-			$message = _receive;
-			return if @{$message} == 0;
+			$message = _receive_nb;
+			return if not $message;
 		}
 
 		for ($message) {
 			$receive->();
 			@save = ($message);
-			redo MESSAGE;
+			next MESSAGE;
 		}
 		continue {
 			return _return_elements($message);
@@ -138,7 +139,7 @@ threads::lite - Actor model threading for Perl
 
 =head1 VERSION
 
-Version 0.029_001
+Version 0.029_002
 
 =head1 SYNOPSIS
 
@@ -283,7 +284,7 @@ L<http://search.cpan.org/dist/threads-lite>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009 Leon Timmermans, all rights reserved.
+Copyright 2009, 2010, 2011 Leon Timmermans, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
